@@ -12,6 +12,7 @@ import fnmatch, errno, threading
 import serial, Queue, select
 import traceback
 import select
+import shlex
 
 from MAVProxy.modules.lib import textconsole
 from MAVProxy.modules.lib import rline
@@ -23,7 +24,7 @@ from MAVProxy.modules.lib import dumpstacks
 try:
       from multiprocessing import freeze_support
       from pymavlink import mavwp, mavutil
-      import wx, matplotlib, HTMLParser
+      import matplotlib, HTMLParser
       try:
             import readline
       except ImportError:
@@ -43,8 +44,6 @@ class MPStatus(object):
         self.counters = {'MasterIn' : [], 'MasterOut' : 0, 'FGearIn' : 0, 'FGearOut' : 0, 'Slave' : 0}
         self.setup_mode = opts.setup
         self.mav_error = 0
-        self.target_system = 1
-        self.target_component = 1
         self.altitude = 0
         self.last_altitude_announce = 0.0
         self.last_distance_announce = 0.0
@@ -109,6 +108,8 @@ class MAVFunctions(object):
         self.param_set = param_set
         self.get_mav_param = get_mav_param
         self.say = say_text
+        # input handler can be overridden by a module
+        self.input_handler = None
 
 class MPState(object):
     '''holds state of mavproxy'''
@@ -135,7 +136,7 @@ class MPState(object):
                         range=(0,100), increment=1, tab='Announcements'),
               MPSetting('distreadout', int, 200, 'Distance Readout', range=(0,10000), increment=1),
 
-              MPSetting('moddebug', int, 0, 'Module Debug Level', range=(0,3), increment=1, tab='Debug'),
+              MPSetting('moddebug', int, opts.moddebug, 'Module Debug Level', range=(0,3), increment=1, tab='Debug'),
               MPSetting('compdebug', int, 0, 'Computation Debug Mask', range=(0,3), tab='Debug'),
               MPSetting('flushlogs', bool, False, 'Flush logs on every packet'),
               MPSetting('requireexit', bool, False, 'Require exit command'),
@@ -146,12 +147,20 @@ class MPState(object):
               MPSetting('rallyalt', int, 90, 'Default Rally Altitude', range=(0,10000), increment=1),
               MPSetting('terrainalt', str, 'Auto', 'Use terrain altitudes', choice=['Auto','True','False']),
               MPSetting('rally_breakalt', int, 40, 'Default Rally Break Altitude', range=(0,10000), increment=1),
-              MPSetting('rally_flags', int, 0, 'Default Rally Flags', range=(0,10000), increment=1)
+              MPSetting('rally_flags', int, 0, 'Default Rally Flags', range=(0,10000), increment=1),
+
+              MPSetting('source_system', int, 255, 'MAVLink Source system', range=(0,255), increment=1, tab='MAVLink'),
+              MPSetting('source_component', int, 0, 'MAVLink Source component', range=(0,255), increment=1),
+              MPSetting('target_system', int, 0, 'MAVLink target system', range=(0,255), increment=1),
+              MPSetting('target_component', int, 0, 'MAVLink target component', range=(0,255), increment=1),
+              MPSetting('state_basedir', str, None, 'base directory for logs and aircraft directories'),
+              MPSetting('allow_unsigned', bool, True, 'whether unsigned packets will be accepted')
             ])
 
         self.completions = {
             "script"         : ["(FILENAME)"],
             "set"            : ["(SETTING)"],
+            "status"         : ["(VARIABLE)"],
             "module"    : ["list",
                            "load (AVAILMODULES)",
                            "<unload|reload> (LOADEDMODULES)"]
@@ -164,6 +173,7 @@ class MPState(object):
 
         # mavlink outputs
         self.mav_outputs = []
+        self.sysid_outputs = {}
 
         # SITL output
         self.sitl_output = None
@@ -175,6 +185,8 @@ class MPState(object):
         self.select_extra = {}
         self.continue_mode = False
         self.aliases = {}
+        import platform
+        self.system = platform.system()
 
     def module(self, name):
         '''Find a public module (most modules are private)'''
@@ -270,7 +282,7 @@ def load_module(modname, quiet=False):
             if mpstate.settings.moddebug > 1:
                 import traceback
                 print(traceback.format_exc())
-    print("Failed to load module: %s" % ex)
+    print("Failed to load module: %s. Use 'set moddebug 3' in the MAVProxy console to enable traceback" % ex)
     return False
 
 def unload_module(modname):
@@ -407,6 +419,12 @@ def process_stdin(line):
     '''handle commands from user'''
     if line is None:
         sys.exit(0)
+
+    # allow for modules to override input handling
+    if mpstate.functions.input_handler is not None:
+          mpstate.functions.input_handler(line)
+          return
+    
     line = line.strip()
 
     if mpstate.status.setup_mode:
@@ -426,11 +444,11 @@ def process_stdin(line):
     if not line:
         return
 
-    args = line.split()
+    args = shlex.split(line)
     cmd = args[0]
     while cmd in mpstate.aliases:
         line = mpstate.aliases[cmd]
-        args = line.split() + args[1:]
+        args = shlex.split(line) + args[1:]
         cmd = args[0]
         
     if cmd == 'help':
@@ -458,7 +476,7 @@ def process_stdin(line):
     try:
         fn(args[1:])
     except Exception as e:
-        print("ERROR in command: %s" % str(e))
+        print("ERROR in command %s: %s" % (args[1:], str(e)))
         if mpstate.settings.moddebug > 1:
             traceback.print_exc()
 
@@ -483,6 +501,9 @@ def process_master(m):
         mpstate.logqueue_raw.put(str(s))
 
     if mpstate.status.setup_mode:
+        if mpstate.system == 'Windows':
+           # strip nsh ansi codes
+           s = s.replace("\033[K","")
         sys.stdout.write(str(s))
         sys.stdout.flush()
         return
@@ -492,6 +513,10 @@ def process_master(m):
     msgs = m.mav.parse_buffer(s)
     if msgs:
         for msg in msgs:
+            sysid = msg.get_srcSystem()
+            if sysid in mpstate.sysid_outputs:
+                  # the message has been handled by a specialised handler for this system
+                  continue
             if getattr(m, '_timestamp', None) is None:
                 m.post_message(msg)
             if msg.get_type() == "BAD_DATA":
@@ -518,6 +543,9 @@ def process_mavlink(slave):
         return
     if mpstate.settings.mavfwd and not mpstate.status.setup_mode:
         for m in msgs:
+            if mpstate.status.watch is not None:
+                if fnmatch.fnmatch(m.get_type().upper(), mpstate.status.watch.upper()):
+                    mpstate.console.writeln('> '+ str(m))
             mpstate.master().write(m.get_msgbuf())
     mpstate.status.counters['Slave'] += 1
 
@@ -546,19 +574,19 @@ def log_writer():
             mpstate.logfile.flush()
             mpstate.logfile_raw.flush()
 
-def open_logs():
-    '''open log files'''
-    if opts.append_log or opts.continue_mode:
-        mode = 'a'
-    else:
-        mode = 'w'
-    logfile = opts.logfile
+# If state_basedir is NOT set then paths for logs and aircraft
+# directories are relative to mavproxy's cwd
+def log_paths():
+    '''Returns tuple (logdir, telemetry_log_filepath, raw_telemetry_log_filepath)'''
     if opts.aircraft is not None:
         if opts.mission is not None:
             print(opts.mission)
             dirname = "%s/logs/%s/Mission%s" % (opts.aircraft, time.strftime("%Y-%m-%d"), opts.mission)
         else:
             dirname = "%s/logs/%s" % (opts.aircraft, time.strftime("%Y-%m-%d"))
+        # dirname is currently relative.  Possibly add state_basedir:
+        if mpstate.settings.state_basedir is not None:
+            dirname = os.path.join(mpstate.settings.state_basedir,dirname)
         mkdir_p(dirname)
         highest = None
         for i in range(1, 10000):
@@ -571,18 +599,31 @@ def open_logs():
         elif os.path.exists(fdir):
             print("Flight logs full")
             sys.exit(1)
-        mkdir_p(fdir)
-        print(fdir)
-        logfile = os.path.join(fdir, 'flight.tlog')
-        mpstate.status.logdir = fdir
-    mpstate.logfile_name = logfile
-    mpstate.logfile = open(logfile, mode=mode)
-    mpstate.logfile_raw = open(logfile+'.raw', mode=mode)
-    print("Logging to %s" % logfile)
+        logname = 'flight.tlog'
+        logdir = fdir
+    else:
+        logname = os.path.basename(opts.logfile)
+        dir_path = os.path.dirname(opts.logfile)
+        if not os.path.isabs(dir_path) and mpstate.settings.state_basedir is not None:
+            dir_path = os.path.join(mpstate.settings.state_basedir,dir_path)
+        logdir = dir_path
 
-    # queues for logging
-    mpstate.logqueue = Queue.Queue()
-    mpstate.logqueue_raw = Queue.Queue()
+    mkdir_p(logdir)
+    return (logdir,
+            os.path.join(logdir, logname),
+            os.path.join(logdir, logname + '.raw'))
+
+
+def open_telemetry_logs(logpath_telem, logpath_telem_raw):
+    '''open log files'''
+    if opts.append_log or opts.continue_mode:
+        mode = 'a'
+    else:
+        mode = 'w'
+    mpstate.logfile = open(logpath_telem, mode=mode)
+    mpstate.logfile_raw = open(logpath_telem_raw, mode=mode)
+    print("Log Directory: %s" % mpstate.status.logdir)
+    print("Telemetry log: %s" % logpath_telem)
 
     # use a separate thread for writing to the logfile to prevent
     # delays during disk writes (important as delays can be long if camera
@@ -605,7 +646,7 @@ def set_stream_rates():
         else:
             rate = mpstate.settings.streamrate2
         if rate != -1:
-            master.mav.request_data_stream_send(mpstate.status.target_system, mpstate.status.target_component,
+            master.mav.request_data_stream_send(mpstate.settings.target_system, mpstate.settings.target_component,
                                                 mavutil.mavlink.MAV_DATA_STREAM_ALL,
                                                 rate, 1)
 
@@ -663,6 +704,9 @@ def periodic_tasks():
                     traceback.print_exception(exc_type, exc_value, exc_traceback,
                                               limit=2, file=sys.stdout)
 
+        # also see if the module should be unloaded:
+        if m.needs_unloading:
+            unload_module(m.name)
 
 def main_loop():
     '''main processing loop'''
@@ -670,6 +714,7 @@ def main_loop():
         for master in mpstate.mav_master:
             send_heartbeat(master)
             if master.linknum == 0:
+                print("Waiting for heartbeat from %s" % master.address)
                 master.wait_heartbeat()
         set_stream_rates()
 
@@ -680,6 +725,8 @@ def main_loop():
             line = mpstate.input_queue.get()
             mpstate.input_count += 1
             cmds = line.split(';')
+            if len(cmds) == 1 and cmds[0] == "":
+                  mpstate.empty_input_count += 1                 
             for c in cmds:
                 process_stdin(c)
 
@@ -696,6 +743,9 @@ def main_loop():
                 rin.append(master.fd)
         for m in mpstate.mav_outputs:
             rin.append(m.fd)
+        for sysid in mpstate.sysid_outputs:
+            m = mpstate.sysid_outputs[sysid]
+            rin.append(m.fd)
         if rin == []:
             time.sleep(0.0001)
             continue
@@ -726,6 +776,14 @@ def main_loop():
                           return
                     continue
 
+            for sysid in mpstate.sysid_outputs:
+                m = mpstate.sysid_outputs[sysid]
+                if fd == m.fd:
+                    process_mavlink(m)
+                    if mpstate is None:
+                          return
+                    continue
+
             # this allow modules to register their own file descriptors
             # for the main select loop
             if fd in mpstate.select_extra:
@@ -782,16 +840,18 @@ if __name__ == '__main__':
                       metavar="DEVICE[,BAUD]", help="MAVLink output port and optional baud rate",
                       default=[])
     parser.add_option("--baudrate", dest="baudrate", type='int',
-                      help="default serial baud rate", default=115200)
+                      help="default serial baud rate", default=57600)
     parser.add_option("--sitl", dest="sitl",  default=None, help="SITL output port")
     parser.add_option("--streamrate",dest="streamrate", default=4, type='int',
                       help="MAVLink stream rate")
     parser.add_option("--source-system", dest='SOURCE_SYSTEM', type='int',
                       default=255, help='MAVLink source system for this GCS')
+    parser.add_option("--source-component", dest='SOURCE_COMPONENT', type='int',
+                      default=0, help='MAVLink source component for this GCS')
     parser.add_option("--target-system", dest='TARGET_SYSTEM', type='int',
-                      default=1, help='MAVLink target master system')
+                      default=0, help='MAVLink target master system')
     parser.add_option("--target-component", dest='TARGET_COMPONENT', type='int',
-                      default=1, help='MAVLink target master component')
+                      default=0, help='MAVLink target master component')
     parser.add_option("--logfile", dest="logfile", help="MAVLink master logfile",
                       default='mav.tlog')
     parser.add_option("-a", "--append-log", dest="append_log", help="Append to log files",
@@ -816,14 +876,19 @@ if __name__ == '__main__':
         default=[],
         help='Load the specified module. Can be used multiple times, or with a comma separated list')
     parser.add_option("--mav09", action='store_true', default=False, help="Use MAVLink protocol 0.9")
+    parser.add_option("--mav20", action='store_true', default=False, help="Use MAVLink protocol 2.0")
     parser.add_option("--auto-protocol", action='store_true', default=False, help="Auto detect MAVLink protocol version")
     parser.add_option("--nowait", action='store_true', default=False, help="don't wait for HEARTBEAT on startup")
     parser.add_option("-c", "--continue", dest='continue_mode', action='store_true', default=False, help="continue logs")
     parser.add_option("--dialect",  default="ardupilotmega", help="MAVLink dialect")
     parser.add_option("--rtscts",  action='store_true', help="enable hardware RTS/CTS flow control")
+    parser.add_option("--moddebug",  type=int, help="module debug level", default=0)
     parser.add_option("--mission", dest="mission", help="mission name", default=None)
     parser.add_option("--daemon", action='store_true', help="run in daemon mode, do not start interactive shell")
     parser.add_option("--profile", action='store_true', help="run the Yappi python profiler")
+    parser.add_option("--state-basedir", default=None, help="base directory for logs and aircraft directories")
+    parser.add_option("--version", action='store_true', help="version information")
+    parser.add_option("--default-modules", default="log,signing,wp,rally,fence,param,relay,tuneopt,arm,mode,calibration,rc,auxopt,misc,cmdlong,battery,terrain,output", help='default module list')
 
     (opts, args) = parser.parse_args()
 
@@ -833,14 +898,28 @@ if __name__ == '__main__':
 
     if opts.mav09:
         os.environ['MAVLINK09'] = '1'
+    if opts.mav20:
+        os.environ['MAVLINK20'] = '1'
     from pymavlink import mavutil, mavparm
     mavutil.set_dialect(opts.dialect)
 
+    #version information
+    if opts.version:
+        import pkg_resources
+        version = pkg_resources.require("mavproxy")[0].version
+        print "MAVProxy is a modular ground station using the mavlink protocol"
+        print "MAVProxy Version: " + version
+        sys.exit(1)
+    
     # global mavproxy state
     mpstate = MPState()
     mpstate.status.exit = False
     mpstate.command_map = command_map
     mpstate.continue_mode = opts.continue_mode
+    # queues for logging
+    mpstate.logqueue = Queue.Queue()
+    mpstate.logqueue_raw = Queue.Queue()
+
 
     if opts.speech:
         # start the speech-dispatcher early, so it doesn't inherit any ports from
@@ -854,8 +933,8 @@ if __name__ == '__main__':
               print("%s" % port)
 
     # container for status information
-    mpstate.status.target_system = opts.TARGET_SYSTEM
-    mpstate.status.target_component = opts.TARGET_COMPONENT
+    mpstate.settings.target_system = opts.TARGET_SYSTEM
+    mpstate.settings.target_component = opts.TARGET_COMPONENT
 
     mpstate.mav_master = []
 
@@ -872,9 +951,10 @@ if __name__ == '__main__':
     # Listen for kill signals to cleanly shutdown modules
     fatalsignals = [signal.SIGTERM]
     try:
-          fatalsignals.append(signal.SIGHUP, signal.SIGQUIT)
+        fatalsignals.append(signal.SIGHUP)
+        fatalsignals.append(signal.SIGQUIT)
     except Exception:
-          pass
+        pass
     if opts.daemon: # SIGINT breaks readline parsing - if we are interactive, just let things die
         fatalsignals.append(signal.SIGINT)
 
@@ -883,6 +963,9 @@ if __name__ == '__main__':
 
     load_module('link', quiet=True)
 
+    mpstate.settings.source_system = opts.SOURCE_SYSTEM
+    mpstate.settings.source_component = opts.SOURCE_COMPONENT
+
     # open master link
     for mdev in opts.master:
         if not mpstate.module('link').link_add(mdev):
@@ -891,9 +974,10 @@ if __name__ == '__main__':
     if not opts.master and len(serial_list) == 1:
           print("Connecting to %s" % serial_list[0])
           mpstate.module('link').link_add(serial_list[0].device)
+    elif not opts.master:
+          wifi_device = '0.0.0.0:14550'
+          mpstate.module('link').link_add(wifi_device)
 
-    # log all packets from the master, for later replay
-    open_logs()
 
     # open any mavlink output ports
     for port in opts.output:
@@ -905,23 +989,39 @@ if __name__ == '__main__':
     mpstate.settings.streamrate = opts.streamrate
     mpstate.settings.streamrate2 = opts.streamrate
 
+    if opts.state_basedir is not None:
+        mpstate.settings.state_basedir = opts.state_basedir
+
     msg_period = mavutil.periodic_event(1.0/15)
     heartbeat_period = mavutil.periodic_event(1)
     heartbeat_check_period = mavutil.periodic_event(0.33)
 
     mpstate.input_queue = Queue.Queue()
     mpstate.input_count = 0
+    mpstate.empty_input_count = 0
     if opts.setup:
         mpstate.rl.set_prompt("")
 
+    # call this early so that logdir is setup based on --aircraft
+    (mpstate.status.logdir, logpath_telem, logpath_telem_raw) = log_paths()
+
     if not opts.setup:
         # some core functionality is in modules
-        standard_modules = ['log', 'wp', 'rally','fence','param','relay',
-                            'tuneopt','arm','mode','calibration','rc','auxopt','misc','cmdlong',
-                            'battery','terrain','output']
+        standard_modules = opts.default_modules.split(',')
         for m in standard_modules:
             load_module(m, quiet=True)
 
+    if opts.console:
+        process_stdin('module load console')
+
+    if opts.map:
+        process_stdin('module load map')
+
+    for module in opts.load_module:
+        modlist = module.split(',')
+        for mod in modlist:
+            process_stdin('module load %s' % mod)
+
     if 'HOME' in os.environ and not opts.setup:
         start_script = os.path.join(os.environ['HOME'], ".mavinit.scr")
         if os.path.exists(start_script):
@@ -938,17 +1038,6 @@ if __name__ == '__main__':
         else:
             print("no script %s" % start_script)
 
-    if opts.console:
-        process_stdin('module load console')
-
-    if opts.map:
-        process_stdin('module load map')
-
-    for module in opts.load_module:
-        modlist = module.split(',')
-        for mod in modlist:
-            process_stdin('module load %s' % mod)
-
     if opts.cmd is not None:
         for cstr in opts.cmd:
             cmds = cstr.split(';')
@@ -959,6 +1048,9 @@ if __name__ == '__main__':
         import yappi    # We do the import here so that we won't barf if run normally and yappi not available
         yappi.start()
 
+    # log all packets from the master, for later replay
+    open_telemetry_logs(logpath_telem, logpath_telem_raw)
+
     # run main loop as a thread
     mpstate.status.thread = threading.Thread(target=main_loop, name='main_loop')
     mpstate.status.thread.daemon = True
