@@ -6,7 +6,7 @@ June 2012
 '''
 
 import time
-import wx
+from wx_loader import wx
 
 try:
     import cv2.cv as cv
@@ -89,8 +89,9 @@ class MPImage():
         self.menu = None
         self.popup_menu = None
 
-        self.in_queue = multiprocessing.Queue()
-        self.out_queue = multiprocessing.Queue()
+        from multiprocessing_queue import makeIPCQueue
+        self.in_queue = makeIPCQueue()
+        self.out_queue = makeIPCQueue()
 
         self.default_menu = MPMenuSubMenu('View',
                                           items=[MPMenuItem('Fit Window', 'Fit Window', 'fitWindow'),
@@ -104,10 +105,10 @@ class MPImage():
     def child_task(self):
         '''child process - this holds all the GUI elements'''
         mp_util.child_close_fds()
-        import wx
+        from wx_loader import wx
         state = self
 
-        self.app = wx.PySimpleApp()
+        self.app = wx.App(False)
         self.app.frame = MPImageFrame(state=self)
         self.app.frame.Show()
         self.app.MainLoop()
@@ -177,8 +178,6 @@ class MPImage():
         self.child.terminate()
         self.child.join()
 
-from PIL import Image
-
 class MPImageFrame(wx.Frame):
     """ The main frame of the viewer
     """
@@ -219,6 +218,7 @@ class MPImagePanel(wx.Panel):
         self.wx_popup_menu = None
         self.popup_pos = None
         self.last_size = None
+        self.done_PIL_warning = False
         state.brightness = 1.0
 
         # dragpos is the top left position in image coordinates
@@ -290,9 +290,17 @@ class MPImagePanel(wx.Panel):
         scaled_image = scaled_image.GetSubImage(rect);
         scaled_image = scaled_image.Rescale(int(rect.width*self.zoom), int(rect.height*self.zoom))
         if state.brightness != 1.0:
-            pimg = mp_util.wxToPIL(scaled_image)
-            pimg = Image.eval(pimg, lambda x: int(x * state.brightness))
-            scaled_image = mp_util.PILTowx(pimg)
+            try:
+                from PIL import Image
+                pimg = mp_util.wxToPIL(scaled_image)
+                pimg = Image.eval(pimg, lambda x: int(x * state.brightness))
+                scaled_image = mp_util.PILTowx(pimg)
+            except Exception:
+                if not self.done_PIL_warning:
+                    print("Please install PIL for brightness control")
+                    self.done_PIL_warning = True
+                # ignore lack of PIL library
+                pass
         self.imagePanel.set_image(scaled_image)
         self.need_redraw = False
 
