@@ -46,7 +46,14 @@ class MPSetting:
             if value < minv or value > maxv:
                 return False
         if self.choice is not None:
-            if value not in self.choice:
+            found = False
+            for v in self.choice:
+                if v.lower() == value.lower():
+                    found = True
+                    value = v
+                    break
+            if not found:
+                print("Must be one of %s" % str(self.choice))
                 return False
         self.value = value
         return True
@@ -95,9 +102,10 @@ class MPSettings(object):
 
 
     def __getattr__(self, name):
-        if name in self._vars:
+        try:
             return self._vars[name].value
-        raise AttributeError
+        except Exception:
+            raise AttributeError
 
     def __setattr__(self, name, value):
         if name[0] == '_':
