@@ -4,6 +4,22 @@
 
 import math
 import os
+import platform
+
+# Some platforms (CYGWIN and others) many not have the wx library
+# use imp to see if wx is on the path
+has_wxpython = False
+
+if platform.system() == 'Windows':
+    # auto-detection is failing on windows, for an unknown reason
+    has_wxpython = True    
+else:
+    import imp
+    try:
+        imp.find_module('wx')
+        has_wxpython = True
+    except ImportError, e:
+        pass
 
 radius_of_earth = 6378100.0 # in meters
 
@@ -149,7 +165,7 @@ def degrees_to_dms(degrees):
     deg = int(degrees)
     min = int((degrees - deg)*60)
     sec = ((degrees - deg) - (min/60.0))*60*60
-    return u'%u\u00b0%02u\'%04.1f"' % (deg, abs(min), abs(sec))
+    return u'%d\u00b0%02u\'%05.2f"' % (deg, abs(min), abs(sec))
 
 
 class UTMGrid:
@@ -199,7 +215,7 @@ def wxToPIL(wimg):
 
 def PILTowx(pimg):
     '''convert a PIL Image to a wx image'''
-    import wx
+    from wx_loader import wx
     wimg = wx.EmptyImage(pimg.size[0], pimg.size[1])
     wimg.SetData(pimg.convert('RGB').tostring())
     return wimg
@@ -233,3 +249,31 @@ def download_files(files):
             open(file, mode='w').write(data)
         except Exception as e:
             print("Failed to save to %s : %s" % (file, e))
+
+
+child_fd_list = []
+
+def child_close_fds():
+    '''close file descriptors that a child process should not inherit.
+       Should be called from child processes.'''
+    global child_fd_list
+    import os
+    while len(child_fd_list) > 0:
+        fd = child_fd_list.pop(0)
+        try:
+            os.close(fd)
+        except Exception as msg:
+            pass
+
+def child_fd_list_add(fd):
+    '''add a file descriptor to list to be closed in child processes'''
+    global child_fd_list
+    child_fd_list.append(fd)
+    
+def child_fd_list_remove(fd):
+    '''remove a file descriptor to list to be closed in child processes'''
+    global child_fd_list
+    try:
+        child_fd_list.remove(fd)
+    except Exception:
+        pass
