@@ -23,10 +23,12 @@ class ConsoleModule(mp_module.MPModule):
         self.total_time = 0.0
         self.speed = 0
         self.max_link_num = 0
+        self.last_sys_status_health = 0
         mpstate.console = wxconsole.MessageConsole(title='Console')
 
         # setup some default status information
         mpstate.console.set_status('Mode', 'UNKNOWN', row=0, fg='blue')
+        mpstate.console.set_status('ARM', 'ARM', fg='grey', row=0)
         mpstate.console.set_status('GPS', 'GPS: --', fg='red', row=0)
         mpstate.console.set_status('Vcc', 'Vcc: --', fg='red', row=0)
         mpstate.console.set_status('Radio', 'Radio: --', row=0)
@@ -35,6 +37,7 @@ class ConsoleModule(mp_module.MPModule):
         mpstate.console.set_status('AS', 'AS', fg='grey', row=0)
         mpstate.console.set_status('RNG', 'RNG', fg='grey', row=0)
         mpstate.console.set_status('AHRS', 'AHRS', fg='grey', row=0)
+        mpstate.console.set_status('EKF', 'EKF', fg='grey', row=0)
         mpstate.console.set_status('Heading', 'Hdg ---/---', row=2)
         mpstate.console.set_status('Alt', 'Alt ---', row=2)
         mpstate.console.set_status('AGL', 'AGL ---/---', row=2)
@@ -55,10 +58,11 @@ class ConsoleModule(mp_module.MPModule):
         mpstate.console.ElevationMap = mp_elevation.ElevationModel()
 
         # create the main menu
-        self.menu = MPMenuTop([])
-        self.add_menu(MPMenuSubMenu('MAVProxy',
-                                    items=[MPMenuItem('Settings', 'Settings', 'menuSettings'),
-                                           MPMenuItem('Map', 'Load Map', '# module load map')]))
+        if mp_util.has_wxpython:
+            self.menu = MPMenuTop([])
+            self.add_menu(MPMenuSubMenu('MAVProxy',
+                                        items=[MPMenuItem('Settings', 'Settings', 'menuSettings'),
+                                               MPMenuItem('Map', 'Load Map', '# module load map')]))
 
     def add_menu(self, menu):
         '''add a new menu'''
@@ -135,9 +139,13 @@ class ConsoleModule(mp_module.MPModule):
                 sats_string = "%u" % num_sats1
             else:
                 sats_string = "%u/%u" % (num_sats1, num_sats2)
-            if ((msg.fix_type == 3 and master.mavlink10()) or
+            if ((msg.fix_type >= 3 and master.mavlink10()) or
                 (msg.fix_type == 2 and not master.mavlink10())):
-                self.console.set_status('GPS', 'GPS: OK (%s)' % sats_string, fg='green')
+                if (msg.fix_type >= 4):
+                    fix_type = "%u" % msg.fix_type
+                else:
+                    fix_type = ""
+                self.console.set_status('GPS', 'GPS: OK%s (%s)' % (fix_type, sats_string), fg='green')
             else:
                 self.console.set_status('GPS', 'GPS: %u (%s)' % (msg.fix_type, sats_string), fg='red')
             if master.mavlink10():
@@ -166,7 +174,11 @@ class ConsoleModule(mp_module.MPModule):
                 if agl_alt is not None:
                     agl_alt = self.settings.basealt - agl_alt
             else:
-                agl_alt_home = self.console.ElevationMap.GetElevation(home_lat, home_lng)
+                try:
+                    agl_alt_home = self.console.ElevationMap.GetElevation(home_lat, home_lng)
+                except Exception as ex:
+                    print(ex)
+                    agl_alt_home = None
                 if agl_alt_home is not None:
                     agl_alt = self.console.ElevationMap.GetElevation(lat, lng)
                 if agl_alt is not None:
@@ -184,13 +196,18 @@ class ConsoleModule(mp_module.MPModule):
             self.console.set_status('GPSSpeed', 'GPSSpeed %u' % msg.groundspeed)
             self.console.set_status('Thr', 'Thr %u' % msg.throttle)
             t = time.localtime(msg._timestamp)
-            if msg.groundspeed > 3 and not self.in_air:
+            flying = False
+            if self.mpstate.vehicle_type == 'copter':
+                flying = self.master.motors_armed()
+            else:
+                flying = msg.groundspeed > 3
+            if flying and not self.in_air:
                 self.in_air = True
                 self.start_time = time.mktime(t)
-            elif msg.groundspeed > 3 and self.in_air:
+            elif flying and self.in_air:
                 self.total_time = time.mktime(t) - self.start_time
                 self.console.set_status('FlightTime', 'FlightTime %u:%02u' % (int(self.total_time)/60, int(self.total_time)%60))
-            elif msg.groundspeed < 3 and self.in_air:
+            elif not flying and self.in_air:
                 self.in_air = False
                 self.total_time = time.mktime(t) - self.start_time
                 self.console.set_status('FlightTime', 'FlightTime %u:%02u' % (int(self.total_time)/60, int(self.total_time)%60))
@@ -202,8 +219,10 @@ class ConsoleModule(mp_module.MPModule):
                         'MAG'  : mavutil.mavlink.MAV_SYS_STATUS_SENSOR_3D_MAG,
                         'INS'  : mavutil.mavlink.MAV_SYS_STATUS_SENSOR_3D_ACCEL | mavutil.mavlink.MAV_SYS_STATUS_SENSOR_3D_GYRO,
                         'AHRS' : mavutil.mavlink.MAV_SYS_STATUS_AHRS,
+                        'RC'   : mavutil.mavlink.MAV_SYS_STATUS_SENSOR_RC_RECEIVER,
                         'TERR' : mavutil.mavlink.MAV_SYS_STATUS_TERRAIN,
                         'RNG'  : mavutil.mavlink.MAV_SYS_STATUS_SENSOR_LASER_POSITION}
+            announce = [ 'RC' ]
             for s in sensors.keys():
                 bits = sensors[s]
                 present = ((msg.onboard_control_sensors_enabled & bits) == bits)
@@ -218,10 +237,36 @@ class ConsoleModule(mp_module.MPModule):
                 if s == 'TERR' and fg == 'green' and master.field('TERRAIN_REPORT', 'pending', 0) != 0:
                     fg = 'yellow'
                 self.console.set_status(s, s, fg=fg)
+            for s in announce:
+                bits = sensors[s]
+                present = ((msg.onboard_control_sensors_enabled & bits) == bits)
+                healthy = ((msg.onboard_control_sensors_health & bits) == bits)
+                was_healthy = ((self.last_sys_status_health & bits) == bits)
+                if present and not healthy and was_healthy:
+                    self.say("%s fail" % s)
+            self.last_sys_status_health = msg.onboard_control_sensors_health
 
         elif type == 'WIND':
             self.console.set_status('Wind', 'Wind %u/%.2f' % (msg.direction, msg.speed))
 
+        elif type == 'EKF_STATUS_REPORT':
+            highest = 0.0
+            vars = ['velocity_variance',
+                    'pos_horiz_variance',
+                    'pos_vert_variance',
+                    'compass_variance',
+                    'terrain_alt_variance']
+            for var in vars:
+                v = getattr(msg, var, 0)
+                highest = max(v, highest)
+            if highest >= 1.0:
+                fg = 'red'
+            elif highest >= 0.5:
+                fg = 'orange'
+            else:
+                fg = 'green'
+            self.console.set_status('EKF', 'EKF', fg=fg)
+
         elif type == 'HWSTATUS':
             if msg.Vcc >= 4600 and msg.Vcc <= 5300:
                 fg = 'green'
@@ -254,6 +299,16 @@ class ConsoleModule(mp_module.MPModule):
             self.console.set_status('Radio', 'Radio %u/%u %u/%u' % (msg.rssi, msg.noise, msg.remrssi, msg.remnoise), fg=fg)
         elif type == 'HEARTBEAT':
             self.console.set_status('Mode', '%s' % master.flightmode, fg='blue')
+            if self.master.motors_armed():
+                arm_colour = 'green'
+            else:
+                arm_colour = 'red'
+            armstring = 'ARM'
+            # add safety switch state
+            if 'SYS_STATUS' in self.mpstate.status.msgs:
+                if (self.mpstate.status.msgs['SYS_STATUS'].onboard_control_sensors_enabled & mavutil.mavlink.MAV_SYS_STATUS_SENSOR_MOTOR_OUTPUTS) == 0:
+                    armstring += '(SAFE)'
+            self.console.set_status('ARM', armstring, fg=arm_colour)
             if self.max_link_num != len(self.mpstate.mav_master):
                 for i in range(self.max_link_num):
                     self.console.set_status('Link%u'%(i+1), '', row=1)
@@ -273,10 +328,15 @@ class ConsoleModule(mp_module.MPModule):
                     if linkdelay > 1:
                         fg = 'orange'
                     else:
-                        fg = 'darkgreen'
+                        fg = 'dark green'
                 self.console.set_status('Link%u'%m.linknum, linkline, row=1, fg=fg)
         elif type in ['WAYPOINT_CURRENT', 'MISSION_CURRENT']:
-            self.console.set_status('WP', 'WP %u' % msg.seq)
+            wpmax = self.module('wp').wploader.count()
+            if wpmax > 0:
+                wpmax = "/%u" % wpmax
+            else:
+                wpmax = ""
+            self.console.set_status('WP', 'WP %u%s' % (msg.seq, wpmax))
             lat = master.field('GLOBAL_POSITION_INT', 'lat', 0) * 1.0e-7
             lng = master.field('GLOBAL_POSITION_INT', 'lon', 0) * 1.0e-7
             if lat != 0 and lng != 0:
@@ -300,7 +360,11 @@ class ConsoleModule(mp_module.MPModule):
                 aspd_error_sign = "L"
             else:
                 aspd_error_sign = "H"
-            self.console.set_status('AltError', 'AltError %d%s' % (msg.alt_error, alt_error_sign))
+            if math.isnan(msg.alt_error):
+                alt_error = "NaN"
+            else:
+                alt_error = "%d%s" % (msg.alt_error, alt_error_sign)
+            self.console.set_status('AltError', 'AltError %s' % alt_error)
             self.console.set_status('AspdError', 'AspdError %.1f%s' % (msg.aspd_error*0.01, aspd_error_sign))
 
 def init(mpstate):
