@@ -32,6 +32,20 @@ joymap = {
      (2, 500, 1500),
      (5, 500, 1500)],
 
+    '*SAILI Simulator*':
+    # Same as above but is reporting a different name
+    # http://www.hobbyking.com/hobbyking/store/__13597__USB_Simulator_Cable_XTR_AeroFly_FMS.html
+    # has 4 usable axes. The last 4 are binary switches and not PWM outputs.
+    # Axis, Scaling Number, 
+    [(0, 500, 1500),
+     (3, 500, 1500),
+     (1, 500, 1500),
+     (2, 500, 1500),
+     (4, 500, 1500),
+     (5, 500, 1500),
+     (6, 500, 1500),
+     (7, 500, 1500)],
+
     'Sony PLAYSTATION(R)3 Controller':
     # only 4 axes usable. This assumes mode 1
     [(2, 500,  1500),
@@ -39,12 +53,23 @@ joymap = {
      (3, -500, 1000),
      (0, -500,  1500)],
 
+    'Microsoft X-Box 360 pad':
+    # This is for a USB cabled X-Box 360 controller
+    # only 4 axes usable. Left and right trigger are ch5 and ch4.
+    # This assumes mode 1
+    [(3, 500,  1500),
+     (1, -500,  1500),
+     (4, -500, 1500),
+     (0, 500,  1500),
+     (2, 500, 1500),
+     (5, 500, 1500)],
+
     'GREAT PLANES InterLink Elite':
     # 4 axes usable
     [(0, 500,  1500),
-     (1, -500,  1500),
+     (1, 500,  1500),
      (2, -1000, 1500),
-     (4, -500,  1500),
+     (4, 500,  1500),
      None,
      None,
      None,
@@ -71,6 +96,18 @@ joymap = {
      None,
      None],
 
+    # Support for Optic 6 with the IPACS easyFly2 Interface USB Adapter from:
+    # http://www.ikarus.net/deutsch-infos-zubehor/die-ikarus-interfacekabel/?lang=en
+    'IPACS easyFly2 Interface':
+    [(0, 758, 1242), # Roll
+     (1, 758, 1242), # Pitch
+     (2, 770, 1230), # Throttle
+     (4, 2553, 974), # (custom: flaps?)
+     None,
+     (3, 758, 1242), # Yaw
+     (5, 758, 1242), # (custom: switch)
+     None],
+
     # This supports a ADC cheap ebay USB controller
     # sold as a "FMS Simulator" joystick
     'ADC':
@@ -79,7 +116,15 @@ joymap = {
      (2, -500, 1500), # Throttle
      (4, 500, 1500), # Yaw
      None,
-     None]
+     None],
+
+    'WAILLY PPM*':
+    [(0, 500, 1500),
+     (1, 500, 1500),
+     (2, 500, 1500),
+     (5, 500, 1500),
+     (3, 500, 1600),
+     (4, 500, 1500)]
 }
 
 class JSModule(mp_module.MPModule):
