@@ -34,8 +34,8 @@ class LinkModule(mp_module.MPModule):
         self.add_completion_function('(SERIALPORT)', self.complete_serial_ports)
         self.add_completion_function('(LINKS)', self.complete_links)
 
+        self.menu_added_console = False
         if mp_util.has_wxpython:
-            self.menu_added_console = False
             self.menu_add = MPMenuSubMenu('Add', items=[])
             self.menu_rm = MPMenuSubMenu('Remove', items=[])
             self.menu = MPMenuSubMenu('Link',
@@ -48,12 +48,16 @@ class LinkModule(mp_module.MPModule):
 
     def idle_task(self):
         '''called on idle'''
-        if not self.menu_added_console and self.module('console') is not None:
+        if mp_util.has_wxpython and (not self.menu_added_console and self.module('console') is not None):
             self.menu_added_console = True
             # we don't dynamically update these yet due to a wx bug
             self.menu_add.items = [ MPMenuItem(p, p, '# link add %s' % p) for p in self.complete_serial_ports('') ]
             self.menu_rm.items = [ MPMenuItem(p, p, '# link remove %s' % p) for p in self.complete_links('') ]
             self.module('console').add_menu(self.menu)
+        for m in self.mpstate.mav_master:
+            m.source_system = self.settings.source_system
+            m.mav.srcSystem = m.source_system
+            m.mav.srcComponent = self.settings.source_component
 
     def complete_serial_ports(self, text):
         '''return list of serial ports'''
@@ -107,7 +111,11 @@ class LinkModule(mp_module.MPModule):
     def link_add(self, device):
         '''add new link'''
         try:
-            conn = mavutil.mavlink_connection(device, autoreconnect=True, baud=self.settings.baudrate)
+            print("Connect %s source_system=%d" % (device, self.settings.source_system))
+            conn = mavutil.mavlink_connection(device, autoreconnect=True,
+                                              source_system=self.settings.source_system,
+                                              baud=self.settings.baudrate)
+            conn.mav.srcComponent = self.settings.source_component
         except Exception as msg:
             print("Failed to connect to %s : %s" % (device, msg))
             return False
@@ -176,6 +184,10 @@ class LinkModule(mp_module.MPModule):
 
     def master_send_callback(self, m, master):
         '''called on sending a message'''
+        if self.status.watch is not None:
+            if fnmatch.fnmatch(m.get_type().upper(), self.status.watch.upper()):
+                self.mpstate.console.writeln('> '+ str(m))
+
         mtype = m.get_type()
         if mtype != 'BAD_DATA' and self.mpstate.logqueue:
             usec = self.get_usec()
@@ -230,14 +242,16 @@ class LinkModule(mp_module.MPModule):
     def master_callback(self, m, master):
         '''process mavlink message m on master, sending any messages to recipients'''
 
+        # see if it is handled by a specialised sysid connection
+        sysid = m.get_srcSystem()
+        if sysid in self.mpstate.sysid_outputs:
+            self.mpstate.sysid_outputs[sysid].write(m.get_msgbuf())
+            return
+
         if getattr(m, '_timestamp', None) is None:
             master.post_message(m)
         self.status.counters['MasterIn'][master.linknum] += 1
 
-        if getattr(m, 'time_boot_ms', None) is not None:
-            # update link_delayed attribute
-            self.handle_msec_timestamp(m, master)
-
         mtype = m.get_type()
 
         # and log them
@@ -248,6 +262,20 @@ class LinkModule(mp_module.MPModule):
             usec = (usec & ~3) | master.linknum
             self.mpstate.logqueue.put(str(struct.pack('>Q', usec) + m.get_msgbuf()))
 
+        # keep the last message of each type around
+        self.status.msgs[m.get_type()] = m
+        if not m.get_type() in self.status.msg_count:
+            self.status.msg_count[m.get_type()] = 0
+        self.status.msg_count[m.get_type()] += 1
+
+        if m.get_srcComponent() == mavutil.mavlink.MAV_COMP_ID_GIMBAL and m.get_type() == 'HEARTBEAT':
+            # silence gimbal heartbeat packets for now
+            return
+
+        if getattr(m, 'time_boot_ms', None) is not None:
+            # update link_delayed attribute
+            self.handle_msec_timestamp(m, master)
+
         if mtype in activityPackets:
             if master.linkerror:
                 master.linkerror = False
@@ -261,11 +289,9 @@ class LinkModule(mp_module.MPModule):
                 return
 
         if mtype == 'HEARTBEAT' and m.get_srcSystem() != 255:
-            if (self.status.target_system != m.get_srcSystem() or
-                self.status.target_component != m.get_srcComponent()):
-                self.status.target_system = m.get_srcSystem()
-                self.status.target_component = m.get_srcComponent()
-                self.say("online system %u component %u" % (self.status.target_system, self.status.target_component),'message')
+            if self.settings.target_system == 0 and self.settings.target_system != m.get_srcSystem():
+                self.settings.target_system = m.get_srcSystem()
+                self.say("online system %u" % self.settings.target_system,'message')
 
             if self.status.heartbeat_error:
                 self.status.heartbeat_error = False
@@ -288,7 +314,8 @@ class LinkModule(mp_module.MPModule):
             if master.flightmode != self.status.flightmode and time.time() > self.status.last_mode_announce + 2:
                 self.status.flightmode = master.flightmode
                 self.status.last_mode_announce = time.time()
-                self.mpstate.rl.set_prompt(self.status.flightmode + "> ")
+                if self.mpstate.functions.input_handler is None:
+                    self.mpstate.rl.set_prompt(self.status.flightmode + "> ")
                 self.say("Mode " + self.status.flightmode)
 
             if m.type == mavutil.mavlink.MAV_TYPE_FIXED_WING:
@@ -377,13 +404,7 @@ class LinkModule(mp_module.MPModule):
 
         if self.status.watch is not None:
             if fnmatch.fnmatch(m.get_type().upper(), self.status.watch.upper()):
-                self.mpstate.console.writeln(m)
-
-        # keep the last message of each type around
-        self.status.msgs[m.get_type()] = m
-        if not m.get_type() in self.status.msg_count:
-            self.status.msg_count[m.get_type()] = 0
-        self.status.msg_count[m.get_type()] += 1
+                self.mpstate.console.writeln('< '+str(m))
 
         # don't pass along bad data
         if mtype != 'BAD_DATA':
