@@ -8,11 +8,12 @@ June 2012
 import sys, os, math
 import functools
 import time
-from MAVProxy.modules.mavproxy_map import mp_slipmap
+from MAVProxy.modules.mavproxy_map import mp_elevation
 from MAVProxy.modules.lib import mp_util
 from MAVProxy.modules.lib import mp_settings
 from MAVProxy.modules.lib import mp_module
 from MAVProxy.modules.lib.mp_menu import *
+from pymavlink import mavutil
 
 class MapModule(mp_module.MPModule):
     def __init__(self, mpstate):
@@ -35,18 +36,25 @@ class MapModule(mp_module.MPModule):
         self.click_time = 0
         self.draw_line = None
         self.draw_callback = None
+        self.have_global_position = False
         self.vehicle_type_name = 'plane'
+        self.ElevationMap = mp_elevation.ElevationModel()
+        self.last_unload_check_time = time.time()
+        self.unload_check_interval = 0.1 # seconds
         self.map_settings = mp_settings.MPSettings(
             [ ('showgpspos', int, 0),
               ('showgps2pos', int, 1),
               ('showsimpos', int, 0),
               ('showahrs2pos', int, 0),
+              ('showahrs3pos', int, 0),
               ('brightness', float, 1),
-              ('rallycircle', bool, False)])
+              ('rallycircle', bool, False),
+              ('loitercircle',bool, False)])
         service='OviHybrid'
         if 'MAP_SERVICE' in os.environ:
             service = os.environ['MAP_SERVICE']
         import platform
+        from MAVProxy.modules.mavproxy_map import mp_slipmap
         mpstate.map = mp_slipmap.MPSlipMap(service=service, elevation=True, title='Map')
         mpstate.map_functions = { 'draw_lines' : self.draw_lines }
     
@@ -58,11 +66,13 @@ class MapModule(mp_module.MPModule):
         self.default_popup = MPMenuSubMenu('Popup', items=[])
         self.add_menu(MPMenuItem('Fly To', 'Fly To', '# guided ',
                                  handler=MPMenuCallTextDialog(title='Altitude (m)', default=100)))
+        self.add_menu(MPMenuItem('Set Home', 'Set Home', '# map sethome '))
         self.add_menu(MPMenuItem('Terrain Check', 'Terrain Check', '# terrain check'))
         self.add_menu(MPMenuItem('Show Position', 'Show Position', 'showPosition'))
 
     def add_menu(self, menu):
         '''add to the default popup menu'''
+        from MAVProxy.modules.mavproxy_map import mp_slipmap
         self.default_popup.add(menu)
         self.mpstate.map.add_object(mp_slipmap.SlipDefaultPopup(self.default_popup, combine=True))
 
@@ -78,10 +88,12 @@ class MapModule(mp_module.MPModule):
             logf = open(os.path.join(self.logdir, "positions.txt"), "a")
             logf.write("Position: %.6f %.6f at %s\n" % (pos[0], pos[1], time.ctime()))
             logf.close()
-        MPMenuChildMessageDialog('Position', msg, font_size=32)
+        posbox = MPMenuChildMessageDialog('Position', msg, font_size=32)
+        posbox.show()
 
     def cmd_map(self, args):
         '''map commands'''
+        from MAVProxy.modules.mavproxy_map import mp_slipmap
         if args[0] == "icon":
             if len(args) < 3:
                 print("Usage: map icon <lat> <lon> <icon>")
@@ -99,11 +111,14 @@ class MapModule(mp_module.MPModule):
         elif args[0] == "set":
             self.map_settings.command(args[1:])
             self.mpstate.map.add_object(mp_slipmap.SlipBrightness(self.map_settings.brightness))
+        elif args[0] == "sethome":
+            self.cmd_set_home(args)
         else:
             print("usage: map <icon|set>")
     
     def display_waypoints(self):
         '''display the waypoints'''
+        from MAVProxy.modules.mavproxy_map import mp_slipmap
         self.mission_list = self.module('wp').wploader.view_list()
         polygons = self.module('wp').wploader.polygon_list()
         self.mpstate.map.add_object(mp_slipmap.SlipClearLayer('Mission'))
@@ -117,19 +132,26 @@ class MapModule(mp_module.MPModule):
                 self.mpstate.map.add_object(mp_slipmap.SlipPolygon('mission %u' % i, p,
                                                                    layer='Mission', linewidth=2, colour=(255,255,255),
                                                                    popup_menu=popup))
+        loiter_rad = self.get_mav_param('WP_LOITER_RAD')
         labeled_wps = {}
+        self.mpstate.map.add_object(mp_slipmap.SlipClearLayer('LoiterCircles'))
         for i in range(len(self.mission_list)):
             next_list = self.mission_list[i]
             for j in range(len(next_list)):
                 #label already printed for this wp?
                 if (next_list[j] not in labeled_wps):
                     self.mpstate.map.add_object(mp_slipmap.SlipLabel(
-                        'miss_cmd %u/%u' % (i,j), polygons[i][j], str(next_list[j]), 'Mission', colour=(0,255,255)))  
+                        'miss_cmd %u/%u' % (i,j), polygons[i][j], str(next_list[j]), 'Mission', colour=(0,255,255)))
+
+                    if (self.map_settings.loitercircle and
+                        self.module('wp').wploader.wp_is_loiter(next_list[j])):
+                        self.mpstate.map.add_object(mp_slipmap.SlipCircle('Loiter Circle %u' % (next_list[j] + 1), 'LoiterCircles', polygons[i][j], abs(loiter_rad), (255, 255, 255), 2))
+
                     labeled_wps[next_list[j]] = (i,j)
-                    
 
     def display_fence(self):
         '''display the fence'''
+        from MAVProxy.modules.mavproxy_map import mp_slipmap
         self.fence_change_time = self.module('fence').fenceloader.last_change
         points = self.module('fence').fenceloader.polygon()
         self.mpstate.map.add_object(mp_slipmap.SlipClearLayer('Fence'))
@@ -306,8 +328,16 @@ class MapModule(mp_module.MPModule):
         self.mpstate.map = None
         self.mpstate.map_functions = {}
     
+    def idle_task(self):
+        now = time.time()
+        if self.last_unload_check_time + self.unload_check_interval < now:
+            self.last_unload_check_time = now
+            if not self.mpstate.map.is_alive():
+                self.needs_unloading = True
+
     def create_vehicle_icon(self, name, colour, follow=False, vehicle_type=None):
         '''add a vehicle to the map'''
+        from MAVProxy.modules.mavproxy_map import mp_slipmap
         if vehicle_type is None:
             vehicle_type = self.vehicle_type_name
         if name in self.have_vehicle and self.have_vehicle[name] == vehicle_type:
@@ -319,6 +349,7 @@ class MapModule(mp_module.MPModule):
     
     def drawing_update(self):
         '''update line drawing'''
+        from MAVProxy.modules.mavproxy_map import mp_slipmap
         if self.draw_callback is None:
             return
         self.draw_line.append(self.click_position)
@@ -328,6 +359,7 @@ class MapModule(mp_module.MPModule):
     
     def drawing_end(self):
         '''end line drawing'''
+        from MAVProxy.modules.mavproxy_map import mp_slipmap
         if self.draw_callback is None:
             return
         self.draw_callback(self.draw_line)
@@ -337,14 +369,33 @@ class MapModule(mp_module.MPModule):
     
     def draw_lines(self, callback):
         '''draw a series of connected lines on the map, calling callback when done'''
+        from MAVProxy.modules.mavproxy_map import mp_slipmap
         self.draw_callback = callback
         self.draw_line = []
         self.mpstate.map.add_object(mp_slipmap.SlipDefaultPopup(None))
+
+    def cmd_set_home(self, args):
+        '''called when user selects "Set Home" on map'''
+        (lat, lon) = (self.click_position[0], self.click_position[1])
+        alt = self.ElevationMap.GetElevation(lat, lon)
+        print("Setting home to: ", lat, lon, alt)
+        self.master.mav.command_long_send(
+            self.settings.target_system, self.settings.target_component,
+            mavutil.mavlink.MAV_CMD_DO_SET_HOME,
+            1, # set position
+            0, # param1
+            0, # param2
+            0, # param3
+            0, # param4
+            lat, # lat
+            lon, # lon
+            alt) # param7
+        
         
     def mavlink_packet(self, m):
         '''handle an incoming mavlink packet'''
+        from MAVProxy.modules.mavproxy_map import mp_slipmap
         if m.get_type() == "HEARTBEAT":
-            from pymavlink import mavutil
             if m.type in [mavutil.mavlink.MAV_TYPE_FIXED_WING]:
                 self.vehicle_type_name = 'plane'
             elif m.type in [mavutil.mavlink.MAV_TYPE_GROUND_ROVER,
@@ -355,9 +406,10 @@ class MapModule(mp_module.MPModule):
                             mavutil.mavlink.MAV_TYPE_COAXIAL,
                             mavutil.mavlink.MAV_TYPE_HEXAROTOR,
                             mavutil.mavlink.MAV_TYPE_OCTOROTOR,
-                            mavutil.mavlink.MAV_TYPE_TRICOPTER,
-                            mavutil.mavlink.MAV_TYPE_HELICOPTER]:
+                            mavutil.mavlink.MAV_TYPE_TRICOPTER]:
                 self.vehicle_type_name = 'copter'
+            elif m.type in [mavutil.mavlink.MAV_TYPE_HELICOPTER]:
+                self.vehicle_type_name = 'heli'
             elif m.type in [mavutil.mavlink.MAV_TYPE_ANTENNA_TRACKER]:
                 self.vehicle_type_name = 'antenna'     
     
@@ -372,7 +424,11 @@ class MapModule(mp_module.MPModule):
         if m.get_type() == "AHRS2" and self.map_settings.showahrs2pos:
             self.create_vehicle_icon('AHRS2' + vehicle, 'blue')
             self.mpstate.map.set_position('AHRS2' + vehicle, (m.lat*1.0e-7, m.lng*1.0e-7), rotation=math.degrees(m.yaw))
-    
+
+        if m.get_type() == "AHRS3" and self.map_settings.showahrs3pos:
+            self.create_vehicle_icon('AHRS3' + vehicle, 'orange')
+            self.mpstate.map.set_position('AHRS3' + vehicle, (m.lat*1.0e-7, m.lng*1.0e-7), rotation=math.degrees(m.yaw))
+
         if m.get_type() == "GPS_RAW_INT" and self.map_settings.showgpspos:
             (lat, lon) = (m.lat*1.0e-7, m.lon*1.0e-7)
             if lat != 0 or lon != 0:
@@ -387,19 +443,32 @@ class MapModule(mp_module.MPModule):
     
         if m.get_type() == 'GLOBAL_POSITION_INT':
             (self.lat, self.lon, self.heading) = (m.lat*1.0e-7, m.lon*1.0e-7, m.hdg*0.01)
-            if self.lat != 0 or self.lon != 0:
+            if abs(self.lat) > 1.0e-3 or abs(self.lon) > 1.0e-3:
+                self.have_global_position = True
                 self.create_vehicle_icon('Pos' + vehicle, 'red', follow=True)
                 self.mpstate.map.set_position('Pos' + vehicle, (self.lat, self.lon), rotation=self.heading)
+
+        if m.get_type() == 'LOCAL_POSITION_NED' and not self.have_global_position:
+            (self.lat, self.lon) = mp_util.gps_offset(0, 0, m.x, m.y)
+            self.heading = math.degrees(math.atan2(m.vy, m.vx))
+            self.create_vehicle_icon('Pos' + vehicle, 'red', follow=True)
+            self.mpstate.map.set_position('Pos' + vehicle, (self.lat, self.lon), rotation=self.heading)
     
         if m.get_type() == "NAV_CONTROLLER_OUTPUT":
-            if self.master.flightmode in [ "AUTO", "GUIDED", "LOITER", "RTL" ]:
+            if (self.master.flightmode in [ "AUTO", "GUIDED", "LOITER", "RTL" ] and
+                self.lat is not None and self.lon is not None):
                 trajectory = [ (self.lat, self.lon),
                                mp_util.gps_newpos(self.lat, self.lon, m.target_bearing, m.wp_dist) ]
                 self.mpstate.map.add_object(mp_slipmap.SlipPolygon('trajectory', trajectory, layer='Trajectory',
-                                                              linewidth=2, colour=(255,0,180)))
+                                                                   linewidth=2, colour=(255,0,180)))
             else:
                 self.mpstate.map.add_object(mp_slipmap.SlipClearLayer('Trajectory'))
-    
+
+        if m.get_type() == "ADSB_VEHICLE":
+            id = 'ADSB-' + str(m.ICAO_address)
+            # use plane icon for now
+            self.create_vehicle_icon(id, 'green', vehicle_type='plane')
+            self.mpstate.map.set_position(id, (m.lat*1e-7, m.lon*1e-7), rotation=m.heading)    
             
         # if the waypoints have changed, redisplay
         last_wp_change = self.module('wp').wploader.last_change
