@@ -9,7 +9,7 @@ from MAVProxy.modules.lib import mp_module
 class ModeModule(mp_module.MPModule):
     def __init__(self, mpstate):
         super(ModeModule, self).__init__(mpstate, "mode")
-        self.add_command('mode', self.cmd_mode, "mode change")
+        self.add_command('mode', self.cmd_mode, "mode change", self.available_modes())
         self.add_command('guided', self.cmd_guided, "fly to a clicked location on map")
 
     def cmd_mode(self, args):
@@ -21,11 +21,22 @@ class ModeModule(mp_module.MPModule):
         if len(args) != 1:
             print('Available modes: ', mode_mapping.keys())
             return
-        mode = args[0].upper()
-        if mode not in mode_mapping:
-            print('Unknown mode %s: ' % mode)
-            return
-        self.master.set_mode(mode_mapping[mode])
+        if args[0].isdigit():
+            modenum = int(args[0])
+        else:
+            mode = args[0].upper()
+            if mode not in mode_mapping:
+                print('Unknown mode %s: ' % mode)
+                return
+            modenum = mode_mapping[mode]
+        self.master.set_mode(modenum)
+
+    def available_modes(self):
+        mode_mapping = self.master.mode_mapping()
+        if mode_mapping is None:
+            print('No mode mapping available')
+            return []
+        return mode_mapping.keys()
 
     def unknown_command(self, args):
         '''handle mode switch by mode name as command'''
@@ -46,6 +57,7 @@ class ModeModule(mp_module.MPModule):
             latitude = float(args[0])
             longitude = float(args[1])
             altitude = int(args[2])
+            latlon = (latitude, longitude)
         else:
             try:
                 latlon = self.module('map').click_position
@@ -58,14 +70,10 @@ class ModeModule(mp_module.MPModule):
             altitude = int(args[0])
             
         print("Guided %s %d" % (str(latlon), altitude))
-        if self.settings.terrainalt:
-            frame = mavutil.mavlink.MAV_FRAME_GLOBAL_TERRAIN_ALT
-        else:
-            frame = mavutil.mavlink.MAV_FRAME_GLOBAL_RELATIVE_ALT
-        self.master.mav.mission_item_send (self.status.target_system,
-                                           self.status.target_component,
+        self.master.mav.mission_item_send (self.settings.target_system,
+                                           self.settings.target_component,
                                            0,
-                                           frame,
+                                           self.module('wp').get_default_frame(),
                                            mavutil.mavlink.MAV_CMD_NAV_WAYPOINT,
                                            2, 0, 0, 0, 0, 0,
                                            latlon[0], latlon[1], altitude)
