@@ -10,7 +10,8 @@ class RelayModule(mp_module.MPModule):
         super(RelayModule, self).__init__(mpstate, "relay")
         self.add_command('relay', self.cmd_relay, "relay commands")
         self.add_command('servo', self.cmd_servo, "servo commands")
-    
+        self.add_command('motortest', self.cmd_motortest, "motortest commands")
+
     def cmd_relay(self, args):
         '''set relays'''
         if len(args) == 0 or args[0] not in ['set', 'repeat']:
@@ -34,7 +35,7 @@ class RelayModule(mp_module.MPModule):
                                                    mavutil.mavlink.MAV_CMD_DO_REPEAT_RELAY, 0,
                                                    int(args[1]), int(args[2]), float(args[3]),
                                                    0, 0, 0, 0)
-    
+
     def cmd_servo(self, args):
         '''set servos'''
         if len(args) == 0 or args[0] not in ['set', 'repeat']:
@@ -58,8 +59,20 @@ class RelayModule(mp_module.MPModule):
                                                    mavutil.mavlink.MAV_CMD_DO_REPEAT_SERVO, 0,
                                                    int(args[1]), int(args[2]), int(args[3]), float(args[4]),
                                                    0, 0, 0)
-            
-              
+
+
+    def cmd_motortest(self, args):
+        '''run motortests on copter'''
+        if len(args) != 4:
+            print("Usage: motortest motornum type value timeout")
+            return
+        self.master.mav.command_long_send(self.target_system,
+                                          0,
+                                          mavutil.mavlink.MAV_CMD_DO_MOTOR_TEST, 0,
+                                          int(args[0]), int(args[1]), int(args[2]), int(args[3]),
+                                          0, 0, 0)
+
+
 def init(mpstate):
     '''initialise module'''
     return RelayModule(mpstate)
