@@ -31,18 +31,18 @@ class WPModule(mp_module.MPModule):
                 self.wploader.load(waytxt)
                 print("Loaded waypoints from %s" % waytxt)
 
+        self.menu_added_console = False
+        self.menu_added_map = False
         if mp_util.has_wxpython:
-            self.menu_added_console = False
-            self.menu_added_map = False
             self.menu = MPMenuSubMenu('Mission',
                                   items=[MPMenuItem('Clear', 'Clear', '# wp clear'),
                                          MPMenuItem('List', 'List', '# wp list'),
                                          MPMenuItem('Load', 'Load', '# wp load ',
-                                                    handler=MPMenuCallFileDialog(flags=wx.FD_OPEN,
+                                                    handler=MPMenuCallFileDialog(flags=('open',),
                                                                                  title='Mission Load',
                                                                                  wildcard='*.txt')),
                                          MPMenuItem('Save', 'Save', '# wp save ',
-                                                    handler=MPMenuCallFileDialog(flags=wx.FD_SAVE|wx.FD_OVERWRITE_PROMPT,
+                                                    handler=MPMenuCallFileDialog(flags=('save', 'overwrite_prompt'),
                                                                                  title='Mission Save',
                                                                                  wildcard='*.txt')),
                                          MPMenuItem('Draw', 'Draw', '# wp draw ',
